# -*- coding: utf-8 -*-

##
## Copyright (C)2005 Ingeniweb
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
$Id: ExFile.py,v 1.42 2006/11/23 09:11:35 b_mathieu Exp $
"""
__docformat__ = 'restructuredtext'
__author__ = ''

from copy import deepcopy

# Zope imports
from AccessControl import ClassSecurityInfo
from ZODB.POSException import ConflictError

# CMF imports
from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

# Archetypes imports
from Products.Archetypes.utils import fixSchema
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

# ATCT imports
from Products.ATContentTypes.content.base import updateActions
from Products.ATContentTypes.content.base import ATCTFileContent
from Products.ATContentTypes.content.file import ATFileSchema, ATFile

# PloneExFile imports
from Products.AttachmentField.AttachmentField import AttachmentField
from Products.AttachmentField.AttachmentWidget import AttachmentWidget

from Products.PloneExFile.interfaces import IExFile
from Products.PloneExFile.config import PROJECTNAME
from Products.PloneExFile import Permissions
from Products.PloneExFile.TTWLockable import TTWLockable
from Products.PloneExFile.fti import ExFileFTI

PloneExFileSchema = ATFileSchema.copy() + Schema((
    AttachmentField(
        'file',
        primary=True,
        searchable=True,
        required=True,
        languageIndependent=True,
        widget=AttachmentWidget(
            label="File",
            label_msgid="label_file",
            description="Select the file to be added by clicking the 'Browse' button.",
            description_msgid="help_file",
            i18n_domain="plone",
            show_content_type=False,
            ),
        ),
    ), marshall=PrimaryFieldMarshaller())

actions = deepcopy(ATFile.actions) + deepcopy(TTWLockable.actions)

class PloneExFile(ATFile, TTWLockable):
    """An external file uploaded to the site."""

    _at_fti_meta_type = ExFileFTI.meta_type

    content_icon   = 'exfile.gif'
    meta_type      = 'ExFile'
    archetype_name = 'File'
    immediate_view = 'base_view'
    default_view   = 'base_view'
    suppl_views    = ()
    typeDescription= 'An external file uploaded to the site.'
    typeDescMsgId  = 'description_edit_file'
    actions = actions
    assocMimetypes = ()
    assocFileExt   = ()
    cmf_edit_kws = ()
    inlineMimetypes= ('application/msword',
                      'application/x-msexcel', # ?
                      'application/vnd.ms-excel',
                      'application/vnd.ms-powerpoint',
                      'application/pdf')

    schema = PloneExFileSchema
    __implements__ = ATFile.__implements__ + (IExFile,)

    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    security.declareProtected(View, 'index_html')
    def index_html(self, REQUEST=None, RESPONSE=None):
        """
            Download the file
        """
        ## Mini-faq:
        ## Why this method override ATFile.index_html?
        ## As of ATCT 1.1.3-final: ATFile.index_html
        ## doesn't pass REQUEST nor RESPONSE to field.download method

        ## Where are the difference?
        ## Just compare the last line.

        field = self.getPrimaryField()
        if field.getContentType(self) in self.inlineMimetypes:
            # return the PDF and Office file formats inline
            return ATCTFileContent.index_html(self, REQUEST, RESPONSE)
        # otherwise return the content as an attachment
        # Please note that text/* cannot be returned inline as
        # this is a security risk (IE renders anything as HTML).

        ## Here, we pass REQUEST and RESPONSE to download().
        return field.download(self, REQUEST, RESPONSE)

    security.declarePublic('getIcon')
    def getIcon(self, relative_to_portal=False):
        """Calculate the icon using the mime type of the file
        """

        field = self.getField('file')
        portal_url = getToolByName( self, 'portal_url' )

        # Find the proper icon
        icon = getattr(self, 'exfile_small.gif')
        m = getattr(field, 'getSmallIcon', None)
        if m is not None:
            icon = m(self)

        # Return it
        if relative_to_portal:
            return portal_url.getRelativeContentURL(icon)

        # Relative to REQUEST['BASEPATH1']
        res = portal_url(relative=1) + '/' + icon.getId()
        while res[:1] == '/':
            res = res[1:]
        return res

registerType(PloneExFile, PROJECTNAME)
