# -*- coding: utf-8 -*-
## PloneExFile
##
## Copyright (C) 2006 Ingeniweb
##
## Portions of this code come from LockManager
## Copyright (C) 2004 Enfold Systems
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
$Id: TTWLockable.py,v 1.2 2006/10/31 11:53:38 b_mathieu Exp $
"""

__docformat__ = 'restructuredtext'
__author__ = ''

from Globals import InitializeClass
from AccessControl import getSecurityManager, ClassSecurityInfo

from webdav.WriteLockInterface import WriteLockInterface
from webdav.LockItem import LockItem

from Products.CMFCore import permissions as CCP
from Products.CMFCore.utils import _checkPermission

from Products.PloneExFile.fti import TTWLockableFTI

ToggleLockPermission = CCP.ModifyPortalContent

class TTWLockable:
    """
    A mixin class for lockable content Through-The-Web. The lock is a WebDav
    Lock.

    Archetypes content type *must* define edit_macros to handle edit properly.
    """
    _at_fti_meta_type = TTWLockableFTI.meta_type

    security = ClassSecurityInfo()

    actions = (
        {
          'id'            : 'unlock',
          'name'          : 'Unlock (Currently locked)',
          'action'        : 'string:${object_url}/toggleLock?action=unlock',
          'permissions'   : (CCP.View, ),
          'condition'     : 'python:object.isLockEnabled() and object.wl_isLocked() and object.canToggleLock()',
          'category'      : 'document_actions',
         },
         {
          'id'            : 'lock',
          'name'          : 'Lock (Currently not locked)',
          'action'        : 'string:${object_url}/toggleLock?action=lock',
          'permissions'   : (CCP.View, ),
          'condition'     : 'python:object.isLockEnabled() and not object.wl_isLocked() and object.canToggleLock()',
          'category'      : 'document_actions',
         },
        )

    security.declareProtected(CCP.View, 'isLockEnabled')
    def isLockEnabled(self):
        ti = self.getTypeInfo()
        return ti.getProperty('lock_enabled')

    security.declareProtected(CCP.View, 'canToggleLock')
    def canToggleLock(self):
        """
        If the content is locked, it can only be unlocked by a lockers or by a
        Manager

        If the content is not locked, the lock permission is sufficient
        """
        return _checkPermission(ToggleLockPermission, self)

    security.declareProtected(ToggleLockPermission, 'toggleLock')
    def toggleLock(self, action=None, REQUEST=None):
        """
        Lock/Unlock content
        """
        security = getSecurityManager()

        if self.wl_isLocked() and action == "unlock": ## already locked and want to unlock
            self.wl_clearLocks()
        elif not self.wl_isLocked() and action == "lock": ## not locked and want to lock
            user = security.getUser()
            ti = self.getTypeInfo()
            timeout = ti.getProperty('lock_timeout')

            lock = LockItem(user, timeout=timeout)
            token = lock.getLockToken()
            self.wl_setLock(token, lock)
        else:
            raise ValueError("No acceptable mode.")

        if REQUEST is not None:
            came_from = REQUEST.get_header('HTTP_REFERER').strip()
            if not came_from:
                print "no referer found!"
                came_from = self.absolute_url()
            return REQUEST.RESPONSE.redirect(came_from)

    security.declareProtected(CCP.ModifyPortalContent, 'canUserEdit')
    def canUserEdit(self):
        """
        Return True if object is not locked or if user is a lock owner
        """
        if not self.wl_isLocked():
            return True

        security = getSecurityManager()
        user = security.getUser()
        return user.getId() in lock_owners(self)

InitializeClass(TTWLockable)

def lock_owners(obj):
    if not WriteLockInterface.isImplementedBy(obj):
        return

    values = obj.wl_lockValues()
    if not values:
        return []

    creators = []
    for lock in values:
        if lock.isValid():
            creator = lock.getCreator()
            if creator:
                # The user id without the path
                creators.append(creator[1])

    if not creators:
        return []

    return creators
