# -*- coding: utf-8 -*-
## PloneExFile
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$Id: fti.py,v 1.2 2006/10/31 11:53:39 b_mathieu Exp $

A specialized Factory Type Information for ExFiles and TTW Lockable contents
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.TypesTool import TypesTool
from Products.CMFDynamicViewFTI.fti import DynamicViewTypeInformation

class TTWLockableFTI(DynamicViewTypeInformation):
    """
    FTI for TTW Lockables.

    Add the properties:
      * use_lock, to enable/disable TTW lock action
      * lock_timeout, in seconds (default: 3600s / 1 hour)
    """

    meta_type = "FTI for TTWLockable content"
    security = ClassSecurityInfo()

    lock_enabled = True
    lock_timeout = 3600
    
    _properties = DynamicViewTypeInformation._properties + (
        {
            'id': 'lock_enabled', 'type': 'boolean', 'mode': 'w',
            'label': 'enable TTW lock?',
            },
        {
            'id': 'lock_timeout', 'type': 'int', 'mode': 'w',
            'label': 'Lock timout (in seconds)',
            },        
        )

InitializeClass(TTWLockableFTI)

def manage_addTTWFTIForm(self, REQUEST):
    """ Get the add form for factory-based type infos.
    """
    addTIForm = DTMLFile('addTypeInfo', _dtmldir).__of__(self)
    ttool = getToolByName(self, 'portal_types')
    return addTIForm( self, REQUEST,
                      add_meta_type=TTWLockableFTI.meta_type,
                      types=ttool.listDefaultTypeInformation() )


class ExFileFTI(TTWLockableFTI):
    """
    FTI for ExFiles and derived content.

    define the property:
    * show_smallpreview
    """

    meta_type = "FTI for ExFile and derived content"
    security = ClassSecurityInfo()

    show_smallpreview = False

    _properties = TTWLockableFTI._properties + (
        {
            'id': 'show_smallpreview', 'type': 'boolean', 'mode': 'w',
            'label': 'Activate small previews',
            },                
        )

InitializeClass(ExFileFTI)

def manage_addExFileFTIForm(self, REQUEST):
    """ Get the add form for factory-based type infos.
    """
    addTIForm = DTMLFile('addTypeInfo', _dtmldir).__of__(self)
    ttool = getToolByName(self, 'portal_types')
    return addTIForm( self, REQUEST,
                      add_meta_type=ExFileFTI.meta_type,
                      types=ttool.listDefaultTypeInformation())


# BBB: the following lines are required to register the new FTI in CMF 1.5 and
# may be removed after switching to CMF 1.6
try:
    from Products.CMFCore.TypesTool import typeClasses
except ImportError:
    pass
else:

    setattr(TypesTool, 'manage_addTTWFTIForm',
                        manage_addTTWFTIForm)

    setattr(TypesTool, 'manage_addTTWFTIForm__roles__',
                        ('Manager', ))

    setattr(TypesTool, 'manage_addExFileFTIForm',
                        manage_addExFileFTIForm)

    setattr(TypesTool, 'manage_addExFileFTIForm__roles__',
                        ('Manager', ))

    typeClasses.append(
        {'class' : TTWLockableFTI,
         'name' : TTWLockableFTI.meta_type,
         'action': 'manage_addTTWFTIForm',
         'permission' : ManagePortal,
         })
    typeClasses.append(
        {'class' : ExFileFTI,
         'name' : ExFileFTI.meta_type,
         'action': 'manage_addExFileFTIForm',
         'permission' : ManagePortal,
         }
        )

